/*
m_ana_clave - A virtual analogue clave

DESCRIPTION
This is a virtual analogue clave, very similar to old drum machines.
NOTE: this UDO requires m_segk.udo and its accompanying ftables
log.ftable and exp.ftable . These ftables are automatically included by this
unit.

SYNTAX
aout m_ana_clave ksize, kdecay, kbright, kaccent, ktrig

INITIALIZATION

PERFORMANCE
ksize - size/pitch (0 to 1)
kdecay - Overall decay (0 to 1)
kbright - Brightness (0 to 1)
kaccent - Accent, if accent >0.5 amplify by 1.25
ktrig - Trigger input

CREDITS
Author: Jeanette C.
*/

opcode m_ana_clave, a, kkkkk
	ksize, klength, kbright, kacc, ktrig xin

	; Set up necessary ftables
	iflog ftgenonce 0, 0, 4096, -23, "log.ftable"
	iflin ftgenonce 0, 0, 4096, 7, 0, 4096, 1
	ifexp ftgenonce 0, 0, 4096, -23, "exp.ftable"
	isine ftgenonce 0, 0, 32768, 10, 1

	; Set real values from 0 to 1 parameter ranges
	kpitch = 800 + (ksize * 4000)
	kdec = .005 + (klength * .045)
	ktone = 2 * kbright

	; Set up envelopes
	kvenv, kvfin m_segk .7, kdec, 0, 1, ktrig, iflog, iflin, ifexp

	; Audio output
	aout foscil kvenv*1.25, kpitch, 1, 3, kvenv*ktone, isine
	aout butterhp aout, kpitch
	aout butterlp aout, kpitch*3*kvenv

	; If kacc >.5 amplify by 1.25
	if kacc >.5 then
		aout *= 1.25
	endif

	xout aout
endop
