/*
m_ana_cowbell - A virtual analogue cowbell

DESCRIPTION
This is a virtual analogue cowbell with control over pitch, tone and brightness.
NOTE: this UDO requires m_segk.udo and its accompanying ftables
log.ftable and exp.ftable . These ftables are automatically included by the
unit.

SYNTAX
aout m_ana_cowbell ksize, kdecay, kbright, ktone, kmetal, kaccent, ktrig

INITIALIZATION

PERFORMANCE
ksize - Size or pitch (0 to 1)
kdecay - Overall decay (0 to 1)
kbright - Brightness or harshness (0 to 1)
ktone - Timbre or tone control (0 to 1)
kmetal - Metallic clarity (0 to 1)
kaccent - Accent, if >0.5 volume is amplified by 1.25
ktrig - Trigger input

CREDITS
Author: Jeanette C.
*/

opcode m_ana_cowbell, a, kkkkkkk
	ksize, klength, kbright, ktone, kmetal, kacc, ktrig xin

	; Set up necessary envelopes
	iflog ftgenonce 0, 0, 4096, -23, "log.ftable"
	iflin ftgenonce 0, 0, 4096, 7, 0, 4096, 1
	ifexp ftgenonce 0, 0, 4096, -23, "exp.ftable"
	isine ftgenonce 0, 0, 32768, 10, 1

	; Set real values from 0 to 1 parameter ranges
	kpitch = 300 + (ksize * 900)
	kdec = .04 + (klength * .096)
	kfmod = kbright * 2
	kdetune = 1.3 + (ktone *.21)

	; Set up envelopes
	kvenv, kvfin m_segk 1, kdec, 0, 0, ktrig, iflog, iflin, ifexp
	kfenv = kvenv * kpitch * (1.5 + kbright) * (1 + (kmetal))

	; Audio output
	atone1 foscil kvenv, kpitch, 1, 1, kfmod, isine
	atone2 foscil kvenv, kpitch*kdetune, 1, 1, kfmod, isine
	aout butterhp (atone1 + atone2)*(.5 - .17 * kmetal), kpitch*1.1
	aout butterlp aout, kfenv

	; If accent >0.5 amplify by 1.25
	if kacc >.5 then
		aout *= 1.25
	endif

	xout aout
endop
