/*
m_ana_cymbal - A virtual analogue cymbal with many tonal controls

DESCRIPTION
This is a virtual analogue cymbal with control over pitch, filter modulation,
envelopes and grit.
NOTE: this UDO requires m_segk.udo and its accompanying ftables
log.ftable and exp.ftable . These ftable are automatically included by this
unit.

SYNTAX
aout m_ana_cymbal ksize, kdecay, ksoft, knoise_mod, kgrit, kbalalnce, \
	kaccent, ktrig

INITIALIZATION

PERFORMANCE
ksize - Size or pitch (0 to 1)
kdecay - Overall decay (0 to 1)
ksoft - Softness of strike (0 to 1)
knoise_mod - Noise filter modulation amount (0 to 1)
kgrit - Grittiness (0 to 1)
kbalalnce - Balance between noise and tonal parts (0 to 1)
kaccent - Accent, if >.5, volume is amplified by 1.25
ktrig - Trigger input

CREDITS
Author: Jeanette C.
*/

opcode m_ana_cymbal, a, kkkkkkkk
	ksize, klength, katt, knmod, kgrit, kbal, kacc, ktrig xin

	; Set up the necessary ftables
	iflog ftgenonce 0, 0, 4096, -23, "log.ftable"
	iflin ftgenonce 0, 0, 4096, 7, 0, 4096, 1
	ifexp ftgenonce 0, 0, 4096, -23, "exp.ftable"
	isine ftgenonce 0, 0, 32768, 10, 1

	; Set real values from 0 to 1 param,eter ranges
	kbasefreq = 200 + (ksize * 1000)
	kbasenoise = kbasefreq * (1 + (ksize * 7))
	kdec = .1 + (klength * .9)
	kport = kdec * ((.1 + katt) * .5)
	knlevel = 3 + (knmod * 6)
	krand = kgrit * 6
	amnoise gauss 1
	acar = 3 + (amnoise * krand)
	amod = 3.01 - (amnoise * krand)
	knbal = .25 + (kbal * .5)
	ktbal = (1 - knbal) * .5

	; Set up necessary envelopes
	kvenv, kvfin m_segk .99, kdec, 0, 2, ktrig, iflog, iflin, ifexp
	kvenv portk kvenv, kport
	kfenv, kffin m_segk (knlevel * kbasefreq), kdec, (kbasefreq), 1, ktrig, iflog, iflin, ifexp
	kfenv portk kfenv, kport*.5, i(kbasefreq)
	kmenv = 1.2 + (kvenv * 4)

	; Audio output
	atone1 foscil 1, kbasefreq, acar, amod, kmenv, isine
	anoise noise kvenv, 0
	atone1 zdf_2pole atone1*kvenv, kbasefreq*7, 1, 1
	afnoise1 zdf_2pole anoise, kfenv, .5, 2
	afnoise2 zdf_2pole anoise*.5, kfenv*1.57, 2, 2
	afnoise3 zdf_2pole anoise*.333, kfenv*2.3, 3, 2
	afnoise butterhp (afnoise1 + afnoise2 + afnoise3), kbasenoise*2
	aout = afnoise*knbal + atone1*ktbal

	; If kacc >.5 amplify by 1.25
	if (kacc >.5) then
		aout *= 1.25
	endif

	xout aout
endop
