/*
m_ana_hihat - A virtual analogue hihat with many tone controls

DESCRIPTION
This is a virtual analogue hihat with control over the noise and metallic
timbre, decay, balance and brightness.
NOTE: This UDO require m_segk.udo and the accompanying ftables
log.ftable and exp.ftable . These ftables will be included automatically
by this unit.

SYNTAX
aout m_ana_hihat ksize, kdecay, ktimbre, kmetal, kbalance, kaccent, ktrig

INITIALIZATION

PERFORMANCE
ksize - Size or pitch of the noise component (0 to 1)
kdecay - Over all decay of the hihat (0 to 1)
ktimbre - Timbre of the metallic tone (0 to 1)
kmetal - Metallic brightness (0 to 1)
kbalance - Balance between tonal and noise component (0 to 1)
kaccent - Accent (if >0.5 volume will be multiplied by 1.25
ktrig - Trigger input

CREDITS
Author: Jeanette C.
*/
opcode m_ana_hihat, a, kkkkkkk
	ksize, klength, ktimbre, kmetal, kbal, kacc, ktrig xin

	; Set up the necessary ftables
	iflog ftgenonce 0, 0, 4096, -23, "log.ftable"
	iflin ftgenonce 0, 0, 4096, 7, 0, 4096, 1
	ifexp ftgenonce 0, 0, 4096, -23, "exp.ftable"
	isine ftgenonce 0, 0, 32768, 10, 1

	; Set real values from the 0 to 1 parameter ranges
	kl = 5 + ((1 - ksize) * 120)
	kbf = 2000 + (ksize * 100)
	ktone = 1.47 + (ktimbre * 1.31)
	kfmamount = 1 + (kmetal * 2)
	kdec = .02 + (klength * .73)
	knbal = .5 + kbal
	ktbal = 2 - knbal

	; Set up envelopes
	kvenv, kvfin m_segk .9, kdec, 0, 1, ktrig, iflog, iflin, ifexp
	kfenv, kffin m_segk kfmamount, kdec*.5, 1, 0, ktrig, iflog, iflin, ifexp
	ktenv, ktfin m_segk .7, kdec*.75, 0, 2, ktrig, iflog, iflin, ifexp

	; Audio output
	; Noise componenets
	anoise noise kvenv, 0
	alnoise nlfilt anoise, .35, -.05, .95, .25, kl
	abnoise butterbp alnoise, kbf, kbf
	ahnoise butterbp alnoise, kbf*2.79, kbf
	anoisy = ahnoise + abnoise

	; Tonal componenets
	atone1 foscil ktenv, kbf*.5, 2, ktone, kfenv, isine
	atone1 butterhp atone1, kbf * (3.5 + (2 * ktimbre))
	amod = atone1 * anoisy
	aout butterhp (anoisy*knbal + atone1*ktbal + amod), kbf * 3

	if kacc <.5 then
		aout *= .8
	endif

	xout aout
endop
