/*
m_ana_rim - A virtual analogue rimshot with many features

DESCRIPTION
This is a virtual analogue rimshot with tonal and noiose components with
separate decays, brightness, an extra bass component and brightness.

SYNTAX
aout m_ana_rim ksize, kdecay, kbass, knoise_decay, knoise_bright, kbright, \
	kbalance, kaccent, ktrig

INITIALIZATION

PERFORMANCE
ksize - Size or pitch of the tonal component (0 to 1)
kdecay - Overall decay (0 to 1)
kbass - Extra tonal bass componenet (0 to 1)
knoise_decay - Noise envelope decay (0 to 1)
knoise_bright - Noise brightness (0 to 1)
kbright - Overall brightness (0 to 1)
kbalance - Balance between tonal and noise parts (0 to 1)
kaccent - Accent, if >0.5 volume will be multiplied by 1.25
ktrig - Trigger input

CREDITS
Author: Jeanette C.
*/

opcode m_ana_rim, a, kkkkkkkkk
	ksize, klength, kbase, knlength, knbright, kbright, kbal, kacc, ktrig xin

	; Set up the necessary ftables
	iflog ftgenonce 0, 0, 4096, -23, "log.ftable"
	iflin ftgenonce 0, 0, 4096, 7, 0, 4096, 1
	ifexp ftgenonce 0, 0, 4096, -23, "exp.ftable"
	irim ftgenonce 0, 0, 32768, 10, 0.971, 0.269, 0.041, 0.054, 0.011, 0.013, 0.08, 0.0065, 0.005, 0.004, 0.003, 0.003, 0.002, 0.002, 0.002, 0.002, 0.002, 0.001, 0.001, 0.001, 0.001, 0.001, 0.002, 0.001, 0.001
	isine ftgenonce 0, 0, 32768, 10, 1

	; Set up real values from 0 to 1 ranges
	kpitch = 500 + (1500 * ksize)
	kdec2 = .001 + (klength * .015) / (1 + ksize)
	kdec = .0005 + (klength * .0075)
	kndec = .002 + (knlength * .398)
	kfilter = 750 + (2250 * knbright)
	kbal *= .5
	kdist = 1 - (kbright * .4)

	; Set up the evenlopes
	kvenv, kvfin m_segk .75, kdec, 0, 2, ktrig, iflog, iflin, ifexp
	kvenv2, kvfin2 m_segk .75, kdec2, 0, 2, ktrig, iflog, iflin, ifexp
	knenv, knfin m_segk .75, kndec, 0, 0, ktrig, iflog, iflin, ifexp
	kfenv, kffin m_segk kfilter, kndec, kfilter * .75, 1, ktrig, iflog, iflin, ifexp

	; Audio output
	atone1 oscil kvenv, kpitch, irim
	atone1 butterbp atone1, kpitch, kpitch*3
	atone2 foscil kvenv2, kpitch*.5, 1, 1, 1, isine
	atone2 butterbp atone2, kpitch/2, kpitch*2
	anoise fractalnoise knenv, 0
	afnoise zdf_2pole anoise, kpitch, 5, 3
	afnoise zdf_2pole afnoise, kfenv, 2, 4
	afnoise butterlp afnoise, 4000
	arim = atone1 * (1 - kbal) + afnoise * kbal + atone2 * kbase * .5
	aout powershape arim, kdist

	if kacc >.5 then
		aout *= 1.25
	endif

	xout aout
endop
