/*
m_ana_shaker - A virtual analogue shaker

DESCRIPTION
This is a virtual analogue shaker with control over size, decay and rattling.
NOTE: this UDO requires m_segk.udo and its accompanying ftables
log.ftable and exp.ftable . These ftables are automatically included by this
unit.

SYNTAX
aout m_ana_shaker ksize, kdecay, krattle, kaccent, ktrig

INITIALIZATION

PERFORMANCE
ksize - Size or highpass cutoff (0 to 1)
kdecay - Overall decay (0 to 1)
krattle - Amount of rattling (0 to 1)
kaccent - Accent, if >0.5 amplify by 1.25
ktrig - Trigger input

CREDITS
Author: Jeanette C.
*/

opcode m_ana_shaker, a, kkkkk
	ksize, klength, krattle, kacc, ktrig xin

	; Set up necessary ftables
	iflog ftgenonce 0, 0, 4096, -23, "log.ftable"
	iflin ftgenonce 0, 0, 4096, 7, 0, 4096, 1
	ifexp ftgenonce 0, 0, 4096, -23, "exp.ftable"

	; Set real values from 0 to 1 parameter ranges
	kpitch = 3000 + (ksize * 3000)
	kdec = .01 + (klength * .49)
	kcount  = int(krattle * 10)

	; Set up envelopes
	kvenv, kvfin m_segk .75, kdec, 0, 2, ktrig, iflog, iflin, ifexp
	kvenvp portk kvenv, kdec*.1
	kvenv2, kvfin2 m_segk 1, kdec*.5, 0, 2, ktrig, iflog, iflin, ifexp

	; Audio output
	afd init 0
	ashake noise kvenvp, 0
	ashake butterhp ashake, kpitch
	anoise noise kvenv2, 0
	anoise butterhp anoise, kpitch+3000

	; The rattle through delays
	if kcount >0 then
		adummy delayr .1
		kndx = 1
		while (kndx <= kcount) do
			adel deltap kndx*.01
			adel butterlp adel*(.75 / ((kcount + 1) - kndx)), (1000 + (200 * kndx))
			afd += adel
			kndx += 1
		od
		delayw anoise
	endif
	aout = ashake*.8 + afd*.5

	; If Accent >.5 amplify by 1.25
	if kacc >.5 then
		aout *= 1.25
	endif

	xout aout
endop
