/*
m_ana_snare - A virtual analogue snare drum with many features

DESCRIPTION
This is a virtual analogue snare drum with a tonal and a noise componenet,
you have control over basic pitch, pitch envelope, tone/noise mix and noise
filtering.
NOTE: this UDO requires m_segk and the accompanying ftables
log.ftable and exp.ftable they will automatically be loaded.

SYNTAX
aout m_ana_snare ksize, kdecay, kmetal, kpitch_amount, kpitch_decay, \
	knoise_decay, knoise_bright, knoise_tone, kbalance, kaccent, ktrig

INITIALIZATION

PERFORMANCE
ksize - Size or basic pitch of the snare (0 to 1)
kdecay - Tonal volume decay (0 to 1)
kmetal - Tonal metallic component (0 to 1)
kpitch_amount - Pitch modulation amount (0 to 1)
kpitch_decay - Pitch envelope decay (0 to 1)
knoise_decay - Noise volume decay (0 to 1)
knoise_bright - Brightness of noise (0 to 1)
knoise_tone - Resonance of filtered noise (0 to 1)
kbalance - Balance between tone and noise (0 tone only, 1 noise only)
kaccent - Accent, if >.5 volume will be increased by 1.25
ktrig - Trigger input

CREDITS
Autho: Jeanette C.
*/

opcode m_ana_snare, a, kkkkkkkkkkk
	ksize, klength, kmetal, kpitch, katt, knlength, ktone, kbright, kbal, kacc, ktrig xin

	; Set up necessary ftables
	iflog ftgenonce 0, 0, 4096, -23, "log.ftable"
	iflin ftgenonce 0, 0, 4096, 7, 0, 4096, 1
	ifexp ftgenonce 0, 0, 4096, -23, "exp.ftable"
	isine ftgenonce 0, 0, 32768, 10, 1

	; Set up real values from ranges 0 to 1
	kbasefreq = 100 + (ksize * 300)
	kdec = .02 + (klength * .18)
	kbw = kbasefreq + 1000 + (2500 * ktone)
	kcentre = kbasefreq + kbw
	kq = 1 + (4 * kbright)
	kndec = .02 + (knlength * .98)
	kpdec = .001 + (.039 * katt)
	kstartpitch = kbasefreq + ((800 - kbasefreq) * kpitch)

	; Set up envelopes
	kvenv, kvfin m_segk .72, kdec, 0, 2, ktrig, iflog, iflin, ifexp
	kpenv, kpfin m_segk kstartpitch, kpdec, kbasefreq, 2, ktrig, iflog, iflin, ifexp
	knenv, knfin m_segk .72, kndec, 0, 2, ktrig, iflog, iflin, ifexp

	; Audio outputs: tonal and noise
	atone1 foscil kvenv, kpenv, 1, 1, kmetal, isine, 0
	anoise fractalnoise knenv, 0
	afnoise zdf_2pole anoise, kcentre, kq, 2
	aout = atone1 *(1 - kbal) + afnoise * kbal

	; If accent control >.5 amplify by 1.25
	if kacc > .5 then
		aout *= 1.25
	endif

	xout aout
endop
