/*
m_ana_tom - A virtual analogue tom with many features

DESCRIPTION
This is a virtual analogue tom with control over basic pitch, pitch envelope,
timbre and a noise component.
NOTE: this UDO requires m_segk.udo and its accompanying ftables:
log.ftable and exp.ftable . These ftables are automatically includes by this
unit.

SYNTAX
aout m_ana_tom ksize, kdecay, kpitch_amount, kpitch_decay, ktone, kfifth, \
	kdistortion, knoise, knoise_decay, kaccent, ktrig

INITIALIZATION

PERFORMANCE
ksize - Size/pitch of the tom (0 to 1)
kdecay - Overall decay (0 to 1)
kpitch_amount - Pitch modulation amount (0 to 1)
kpitch_decay - Pitch envelope decay (0 to 1)
ktone - Timbre control (0 to 1)
kfifth - Volume of a fifth note abobve base pitch (0 to 1)
kdistortion - Distortion amount (0 to 1)
knoise - Noise component volume (0 to 1)
knoise_decay - Noise component volume decay (0 to 1)
kaccent - Accent, if >0.5 volume will be multiplied by 1.25
ktrig - Trigger input

CREDITS
Author: Jeanette C.
*/

opcode m_ana_tom, a, kkkkkkkkkkk
	ksize, klength, kpitch, kptim, ktone, kfifth, kbright, knoise, knd, kacc, ktrig xin

	; Set up the necessary ftables
	iflog ftgenonce 0, 0, 4096, -23, "log.ftable"
	iflin ftgenonce 0, 0, 4096, 7, 0, 4096, 1
	ifexp ftgenonce 0, 0, 4096, -23, "exp.ftable"
	isine ftgenonce 0, 0, 32768, 10, 1

	; Set real values from the 0 to 1 parameter ranges
	kbasefreq = 80 + (ksize * 320)
	kdec = .05 + (klength * .35)
	kstartpitch = kbasefreq + ((1600 - kbasefreq) * kpitch)
	kpdec = .005 + ((kdec - .005) * kptim)
	kfmod = ktone * 1.5
	kv2 = kfifth
	kharsh = 1 + (1.5 * kbright)
	khcomp = 1 + (kbright * 1.2)
	knv = knoise*.5
	kndec = .05 + (kdec * knd)

	; Set up the necessary envelopes
	kvenv, kvfin m_segk .5, kdec, 0, 0, ktrig, iflog, iflin, ifexp
	keenv, kefin m_segk .2, .01, 0, 2, ktrig, iflog, iflin, ifexp
	knenv, knfin m_segk knv, kndec, 0, 1, ktrig, iflog, iflin, ifexp
	kpenv, kpfin m_segk kstartpitch, kpdec, kbasefreq, 1, ktrig, iflog, iflin, ifexp

	; Audio output
	atone1 foscil kvenv+keenv, kpenv, 1, 1, kfmod, isine
	atone2 foscil kvenv+keenv, kpenv*1.49, 1, 1, kfmod, isine
	atones = atone1 * (1 - (kv2 * .5)) + atone2 *(.5 * kv2)
	atones powershape atones, kharsh
	atones butterlp atones*khcomp, (kpenv * 6 + keenv * kstartpitch * 10)
	anoise noise knenv, .9
	anoise butterbp anoise, kbasefreq*1.5, kbasefreq*1.5
	aout = atones + anoise

	if kacc >.5 then
		aout *= 1.25
	endif

	xout aout
endop
